/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2014 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmtfsapi.h
 *  \brief Header for the Transactional File Server (TFS) API
 */

#include "rdmtfstypes.h"
#include "rdmtypes.h"

/**
 * \addtogroup rdm
 * @{
 * \defgroup tfs RDM TFS APIs
 *
 * The RDM TFS APIs.  Please see the respective sub-modules for details.
 *
 *  A TFS handle can be instantiated using rdm_rdmAllocTFS().
 *
 * @}
 * \addtogroup tfs
 * @{*/

/** \defgroup tfs_initialization TFS Initialization functions
 *
 *     The TFS instantiation API.  The functions here are located in \lib{tfs}
 *
 * \defgroup tfs_cleanup TFS Cleanup Functions
 *
 *     The TFS cleanup API.  The functions here are located in \lib{tfs}
 *
 * \defgroup tfs_public TFS Functions
 *
 *     The TFS function API.  The functions here are located in \lib{tfs}
 *
 * \defgroup tfs_config TFS Configuration Functions
 *
 *     The TFS configuration API.  The functions here are located in \lib{tfs}
 */

/** @}*/

#if defined(RDM_UNICODE)
#define rdm_tfsAllocEncrypt RDM_T_FUNC (rdm_tfsAllocEncrypt)
#define rdm_tfsDropDatabase RDM_T_FUNC (rdm_tfsDropDatabase)
#define rdm_tfsGetConnections RDM_T_FUNC (rdm_tfsGetConnections)
#define rdm_tfsGetOption RDM_T_FUNC (rdm_tfsGetOption)
#define rdm_tfsGetOptions RDM_T_FUNC (rdm_tfsGetOptions)
#define rdm_tfsKillAllRemoteConnections                                        \
    RDM_T_FUNC (rdm_tfsKillAllRemoteConnections)
#define rdm_tfsKillRemoteConnection RDM_T_FUNC (rdm_tfsKillRemoteConnection)
#define rdm_tfsPing RDM_T_FUNC (rdm_tfsPing)
#define rdm_tfsSetIniValue RDM_T_FUNC (rdm_tfsSetIniValue)
#define rdm_tfsSetOption RDM_T_FUNC (rdm_tfsSetOption)
#define rdm_tfsSetOptions RDM_T_FUNC (rdm_tfsSetOptions)
#define rdm_tfsGetCapability RDM_T_FUNC (rdm_tfsGetCapability)
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/** \brief Allocate memory for a new RDM db

    This function allocates memory for and initializes an ::RDM_DB using the
    specified ::RDM_TFS. You must call rdm_dbOpen() or rdm_dbSetCatalog() to
   associate a schema with the database handle.

    \retval{sOKAY}

    \ingroup tfs_initialization

    \see rdm_rdmAllocTFS
    \see rdm_dbFree
    \see rdm_dbOpen
    \see rdm_rdmAllocDatabase
    \see rdm_tfsFree
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsAllocDatabase (
    RDM_TFS tfs, /**< [in] The RDM_TFS associated with the RDM_DB */
    RDM_DB *pDb  /**< [out] A pointer an RDM_DB */
);

/** \brief Allocate an encryption context

    This function allocates and initializes an encryption context. The
    encryption context will be associated with the TFS and will be automatically
    used by any database associated with that TFS.  The encryption handle
    can be associated with a specific database using rdm_dbSetEncrypt().  A
   encryption context associated with a specific database will be used to
   encrypt a database when it is created.  In order to open an already encrypted
   database the runtime engine will first use an encryption context associated
   with the database handle, if that encryption context is not correct it will
   try each encryption context associated with the TFS.

    \par Encryption Ciphers

    RDM supports an implementation of the Advanced Encryption Standard (AES)
    specification and a simple XOR cipher. The XOR cipher encrypts the data by
    applying the bitwise XOR operator to every character using a given passcode.
    The AES cipher allows the user to specify a key size of 128, 192, or 256
    bits. As a symmetric-key block cipher the number of bits of input match the
    number of bits of encrypted data returned by the AES implementation.

    AES encryption is not available on all versions of RDM.

    \par Passcodes

    The encryption passcode contains the optional encryption type and the
    security key used for generating encryption key.

    \code
    passcode:
        [enc_type':']security_key

    enc_type:
            AES128 | AES192 | AES256
        |   aes128 | aes192 | aes256
    \endcode

    If the encryption type (enc_type) is omitted, the default encryption key
    will be AES128 for installations that support AES encryption and XOR for
    those that do not.

    There is no error checking on the encryption type string. If an unrecognized
    encryption type is provided, the entire passcode string will be used as the
    security key with the default encryption type.

    The security key (security_key) can be any NULL-terminated character string
    of any length.

    \par Locking Requirements
    None

    \retval{sOKAY}

    \see rdm_encryptGetType
    \see rdm_dbSetEncrypt
    \see rdm_dbEncrypt
    \see rdm_encryptFree

    \ingroup rdm_encryption
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsAllocEncrypt (
    RDM_TFS hTFS, /**< [in]  TFS handle */
    const RDM_TCHAR_T
        *passcode, /**< [in] Encryption passcode. Can be an empty string but cannot be NULL */
    RDM_ENCRYPT *pEnc /**< [out] Pointer to an encryption context */
);

/** \brief Stop the listener process for the TFS

    This function will stop the listener process for the specified TFS. Stopping
   the listener process prevents new remote connections to TFS from being
   processed.

    \note This function does not terminate remote connections to the TFS that
   have already been established.

    \retval{sOKAY}

    \see rdm_tfsEnableListener
    \see rdm_tfsFree

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsDisableListener (
    RDM_TFS hTFS /**< [in]  TFS handle */
);

/** \brief Drop the specified database

    This function drops the \e entire contents of the database, including
    - The database content files
    - The database schema files
    - The database log files
    - All SQL related files
    - The database directory

    It will fail if the database is open by any other users. This operation is
    not transactional and cannot be undone or rolled back.

    This function is primarily used to maintain temporary or working databases
    that exist only for the duration of the application program.

    \retval{sOKAY}
    \retval{eDBOPENED}
    \retval{sNODB}

    \see rdm_rdmAllocTFS
    \see rdm_tfsSetOptions
    \see rdm_tfsAllocDatabase
    \see rdm_tfsFree

    \ingroup tfs_cleanup
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsDropDatabase (
    RDM_TFS tfs,                  /**< [in] A TFS handle */
    const RDM_TCHAR_T *dbNameSpec /**< [in] The database name specification for
                             the database to be dropped */
);

/** \brief Starts the listener process for the TFS

    This function will enable the listener process for the specified TFS. The
   listener process allows new remote connections to TFS to be processed.

    \retval{sOKAY}

    \see rdm_rdmAllocTFS
    \see rdm_tfsSetOptions
    \see rdm_tfsDisableListener
    \see rdm_tfsFree

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsEnableListener (
    RDM_TFS hTFS /**< [in]  TFS handle */
);

/** \brief Terminate a TFS service

    \retval{sOKAY}

    \see rdm_rdmAllocTFS

    \ingroup tfs_cleanup
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsFree (
    RDM_TFS hTFS /**< [in]  The TFS instance */
);

/** \brief Get memory usage associated with the TFS

    This function returns the current and maximum memory usage of the TFS
    and all database and database objects associated with the TFS.

    \note Untill further notice this function will report memory usage
    for all TFS handles and its databases. This semantic is subject to
    change.

    \retval{sOKAY}

    \see rdm_rdmAllocTFS
    \see rdm_rdmAllocTFSWithMemBuf

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsGetMemUsage (
    RDM_TFS tfs,        /**< [in]  TFS handle */
    size_t *curr_usage, /**< [out] The current memory usage */
    size_t *max_usage   /**< [out] The maximum memory usage */
);

/** \brief Get the list of users connected to the TFS

    This function returns an array of user IDs, each of which represents
    a TFS connection.

    \retval{sOKAY}

    \see rdm_rdmAllocTFS
    \see rdm_tfsKillRemoteConnection
    \see rdm_tfsFree

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsGetConnections (
    RDM_TFS hTFS,           /**< [in]  TFS handle */
    const RDM_TCHAR_T *uri, /**< [in]  Connection URI for remote TFS */
    const RDM_TCHAR_T *const **ppszUserIDs, /**< [out] Array of user IDs */
    uint32_t *puUserCount                   /**< [out] Number of user IDs */
);

/** \brief Get the encryption context from a TFS

    \retval{sOKAY}

    \see rdm_rdmAllocTFS
    \see rdm_tfsAllocEncrypt
    \see rdm_tfsFree

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsGetEncrypt (
    RDM_TFS hTFS,        /**< [in]  TFS handle */
    RDM_ENCRYPT *pEncCtx /**< [out] encryption context */
);

/** \brief Get a single TFS option into a string

    \copydetails rdm_dbGetOption

    \retval{sOKAY}
    \retval{eINVOPTION}
    \retval{eRANGE}

    \see rdm_tfsSetOption

    \see rdm_rdmAllocTFS
    \see rdm_tfsGetEncrypt
    \see rdm_tfsGetOptions
    \see rdm_tfsSetOptions
    \see rdm_tfsFree

    \ingroup tfs_config
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsGetOption (
    RDM_TFS tfs,                /**< [in]  A valid RDM_TFS */
    const RDM_TCHAR_T *keyword, /**< [in]  single option keyword */
    RDM_TCHAR_T *optValue,      /**< [out] option value in string */
    size_t bytesIn,             /**< [in]  # of bytes for pszValue */
    size_t *bytesOut /**< [out] A pointer to the number of bytes returned in optValue (can be NULL) */
);

/** \brief Get TFS options

    \copydetails rdm_dbGetOptions

    \retval{sOKAY}
    \retval{eINVOPTION}
    \retval{eRANGE}

    \see rdm_rdmAllocTFS
    \see rdm_tfsGetOptions
    \see rdm_tfsSetOptions
    \see rdm_tfsFree

    \ingroup tfs_config
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsGetOptions (
    RDM_TFS tfs,            /**< [in]  A valid RDM_TFS */
    RDM_TCHAR_T *optString, /**< [out] option string */
    size_t bytesIn,         /**< [in]  Number of bytes for \a optString */
    size_t *bytesOut /**< [out] A pointer to the number of bytes returned in \a optString (can be NULL) */
);

/** \brief Get the TFS type and version

    This function will obtain the type and version of a specified TFS

    \retval{sOKAY}

    \see rdm_rdmAllocTFS
    \see rdm_tfsFree

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsGetVersion (
    RDM_TFS hTFS,      /**< [in]  TFS handle */
    uint16_t *pMajorV, /**< [out] major version */
    uint16_t *pMinorV, /**< [out] minor version */
    TFS_TYPE *pTfsType /**< [out] TFS type */
);

/** \brief Kill all the TFS connections

    \retval{sOKAY}

    \see rdm_rdmAllocTFS
    \see rdm_tfsGetConnections
    \see rdm_tfsKillRemoteConnection
    \see rdm_tfsPing
    \see rdm_tfsFree

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsKillAllRemoteConnections (
    RDM_TFS tfs,           /**< [in]  TFS handle */
    const RDM_TCHAR_T *uri /**< [in]  Connection URI for TFSR */
);

/** \brief Kill the TFS connection associated with a user

    \retval{sOKAY}

    \see rdm_tfsKillAllRemoteConnections
    \see rdm_tfsPing

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsKillRemoteConnection (
    RDM_TFS hTFS,               /**< [in]  TFS handle */
    const RDM_TCHAR_T *uri,     /**< [in]  Connection URI for TFSR */
    const RDM_TCHAR_T *dbUserID /**< [in]  User whose connection to kill */
);

/** \brief Ping a remote TFS

    This function will ping the TFS specified by the URI to see if the TFS
    is active.

    \retval{sOKAY}

    \see rdm_rdmAllocTFS
    \see rdm_tfsKillAllRemoteConnections

    \ingroup tfs_public
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsPing (
    RDM_TFS hTFS,          /**< [in]  TFS handle */
    const RDM_TCHAR_T *uri /**< [in]  Connection URI for TFSR */
);

/** \brief Set a single TFS option from a string

    This function sets a single TFS option from a string value. The \a keyword
    parameter accepts the key for the option to set. The options are inherited
   by the database and/or SQL handle derived from this TFS handle.

    \includedoc dox/sqldb-options.mdinc

    \retval{sOKAY}
    \retval{eINVOPTION}
    \retval{eINVOPTIONVALUE}

    \see rdm_rdmAllocTFS
    \see rdm_tfsGetOption
    \see rdm_tfsSetOptions

    \ingroup tfs_config
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsSetOption (
    RDM_TFS tfs,                /**< [in] A valid RDM_TFS */
    const RDM_TCHAR_T *keyword, /**< [in] Single option keyword */
    const RDM_TCHAR_T *strValue /**< [in]  option value in string */
);

/** \brief Set TFS options

    This function is used to set any RDM TFS options. The \a optString is a
    key/value pair in the form of "key=value". The options are inherited by
    the database and/or SQL handle derived from this TFS handle.

    \includedoc dox/sqldb-options.mdinc

    \retval{sOKAY}
    \retval{eINVOPTION}
    \retval{eINVOPTIONVALUE}

    \see rdm_rdmAllocTFS
    \see rdm_tfsSetOption
    \see rdm_tfsGetOptions

    \ingroup tfs_config
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsSetOptions (
    RDM_TFS tfs,                 /**< [in]  A valid TFS handle */
    const RDM_TCHAR_T *optString /**< [in]  The option string */
);

/** \brief Get a TFS capability

    This function retrieves a single TFS capability into a boolean value. The
    \a name parameter accepts the name for the capability to get.

    \retval{sOKAY}
    \retval{eINVOPTION}

    \ingroup tfs_config
*/
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_tfsGetCapability (
    RDM_TFS tfs,             /**< [in]  A valid RDM_TFS */
    const RDM_TCHAR_T *name, /**< [in]  single option keyword */
    RDM_BOOL_T *value        /**< [out] option value in string */
);

#if defined(__cplusplus)
}
#endif
